##################################################### #
#         Monte Carlo Experiment  (LC-MNL)
# This script run the monte carlo experiment for Latent Class Models with 2 classes
# By: Mauricio Sarrias
# Date: 22 March, 2016
# Last Modification: September 17, 2017 (For review)
###################################################### #

# Common setting for all the estimations ====
rm(list = ls(all = TRUE))
library("gmnl")
library("mlogit")
source('R Script/boot.lc.R') ## Function to compute K&R procedure

# Function to compute time ====
my.time <- function(object){
  et <- object[3]
  s <- round(et,0)
  h <- s %/% 3600
  s <- s - 3600*h
  m <- s %/% 60
  s <- s - 60*m
  paste(h, "h:", m, "m:", s, "s", sep = "")
}

# Set seed
set.seed(1234)

#=======================================================#
# Experiment 1: N = 300, Boot = 10000 , Classes = 2 ----
#=======================================================#

## Globals
N <- 300                      # Number of individuals as Revelt & Train (2000)
J <- 3                        # Number of alternatives as Revelt & Train (2000)
times <- c(1, 5, 10, 20, 50)  # Choice situation per individual as Revelt & Train (2000)
S <- 300                      # Number of databases
Q <- 1000                     # Number of Boot for Krisky & Robb procedure.


## Generate True Random Coefficients
b1_e1 <- c(rep(-2,   N * 0.6),   rep(2,   N * 0.4))
b2_e1 <- c(rep(-0.5, N * 0.6),   rep(0.5, N * 0.4))

## Matrix for results b11 class 1 var 1, b12 class 1, var2
b11.sim.e1  <- matrix(NA, nrow = S, ncol = length(times))
b12.sim.e1  <- matrix(NA, nrow = S, ncol = length(times))
b21.sim.e1  <- matrix(NA, nrow = S, ncol = length(times))
b22.sim.e1  <- matrix(NA, nrow = S, ncol = length(times))
con2.sim.e1 <- matrix(NA, nrow = S, ncol = length(times))
share1.e1 <- matrix(NA, nrow = S, ncol = length(times))
share2.e1 <- matrix(NA, nrow = S, ncol = length(times))

bi1.sim.e1 <- array(NA, dim = c(S, N, ncol = length(times)))
bi2.sim.e1 <- array(NA, dim = c(S, N, ncol = length(times)))

si1.sim.e1 <- array(NA, dim = c(S, N, ncol = length(times)))
si2.sim.e1 <- array(NA, dim = c(S, N, ncol = length(times)))

bi1b.sim.e1 <- array(NA, dim = c(S, N, ncol = length(times)))
bi2b.sim.e1 <- array(NA, dim = c(S, N, ncol = length(times)))

si1b.sim.e1 <- li1b.sim.e1 <- ui1b.sim.e1  <- array(NA, dim = c(S, N, ncol = length(times)))
si2b.sim.e1 <- li2b.sim.e1 <- ui2b.sim.e1  <- array(NA, dim = c(S, N, ncol = length(times)))

mix.time.e1  <- matrix(NA, nrow = S, ncol = length(times))
boot.time.e1 <- matrix(NA, nrow = S, ncol = length(times))
mess.e1      <- matrix(NA, nrow = S, ncol = length(times))


## Monte Carlo Loop
for (c in 1:length(times)) {
  # Choice situation
  t <- times[c]
  # id for individuals
  id <- rep(1:N, each = J * t)
  
  for (s in 1:S) {
    cat("\n","===========================", "\n")
    cat("Time: ", t, ", and database: ", s, "\n")
    cat("================================", "\n")
    
    # Generate variables that depend on T
    x1 <- rnorm(N * J * t)
    x2 <- as.numeric(runif(N * J * t) < 0.5)

    # The true data generating process
    U <-  b1_e1[id] * x1 + b2_e1[id] * x2 - log(-log(runif(N * J * t))) 
    
    ## Chosen alternative
    chosen <- rep(0, N * J * t)
    for (i in 1:(N * t)) {
      U_j <- U[((i - 1) * J + 1):(i * J)]
      U_max <- max(U_j)
      pos <- which(U_j == U_max)
      chosen[((i - 1) * J + 1):(i * J)][pos] <- TRUE
    }
    
    ## Data format
    data <- as.data.frame(cbind(id, chosen, x1, x2))
    data <- mlogit.data(data, choice = "chosen", shape = "long", 
                        alt.levels = c("1", "2", "3"), id.var = "id")
    # Estimate LC Model
    stm <- proc.time()
    if (t == 1) {
      lc <- gmnl(chosen ~ x1 + x2 | 0 | 0 | 0 | 1, 
                  data = data,
                  model = "lc",
                  print.init = TRUE,
                  Q = 2,  
                  seed = NULL,
                  iterlim = 1000)
    } else {
      lc <- gmnl(chosen ~ x1 + x2  | 0 | 0 | 0 | 1, 
                 data = data,
                 model = "lc",
                 print.init = TRUE,
                 Q = 2,  
                 seed = NULL,
                 iterlim = 1000,
                 panel = TRUE)
    }
    mix.time.e1[s, c] <- (proc.time() - stm)[3]
    cat("\n", "Estimation took: ", my.time(proc.time() - stm), "\n")
    
    cat("\n", "Coefficients are : ", "\n")
    print(coef(lc))
    mess.e1[s, c] <- lc$logLik$message
    
    cat("\n", "How is my LL doing?: ", mess.e1[s, c], "\n")
    b11.sim.e1[s, c] <- coef(lc)[1]
    b12.sim.e1[s, c] <- coef(lc)[2]
    b21.sim.e1[s, c] <- coef(lc)[3]
    b22.sim.e1[s, c] <- coef(lc)[4]
    con2.sim.e1[s, c] <- coef(lc)[5]
    
    ## Compute probability for each class
    share1.e1[s, c] <- exp(0) / (exp(0) + exp(con2.sim.e1[s, c]))
    share2.e1[s, c] <- exp(con2.sim.e1[s, c]) / (exp(0) + exp(con2.sim.e1[s, c]))
    
    ## Compute individual-estimates
    bq  <- lc$bi
    Qiq <- lc$Qir
    N   <- nrow(Qiq)
    K   <- ncol(bq)

    mean <- matrix(NA, N, K)
    se   <- matrix(NA, N, K)
    colnames(mean) <- colnames(se) <- colnames(bq)
    
    for (j in 1:K) {
      b_temp <- gmnl:::repRows(bq[, j], N)
      mean[, j] <- rowSums(b_temp * Qiq)
      se[, j]   <- sqrt(rowSums((b_temp - mean[, j])^2 * Qiq))
    }
    
    
    bi1.sim.e1[s, , c] <- mean[, "x1"]
    bi2.sim.e1[s, , c] <- mean[, "x2"]
    si1.sim.e1[s, , c] <- se[, "x1"]
    si2.sim.e1[s, , c] <- se[, "x2"]
    
    # K&R procedure
    sbt <- proc.time()
    tryCatch({
      if (t == 1) {
        bootS <- boot.lc(lc, panel =  FALSE, S = Q, Q = 2) 
      } else {
        bootS <- boot.lc(lc, panel =  TRUE,  S = Q, Q = 2) 
      } 
      bi1b.sim.e1[s, , c] <- bootS$mean[ ,"x1"]
      bi2b.sim.e1[s, , c] <- bootS$mean[ ,"x2"]
      si1b.sim.e1[s, , c] <- bootS$se[ ,"x1"]
      si2b.sim.e1[s, , c] <- bootS$se[ ,"x2"]
      li1b.sim.e1[s, , c] <- bootS$quant[1, ,"x1"]
      ui1b.sim.e1[s, , c] <- bootS$quant[2, ,"x1"]
      li2b.sim.e1[s, , c] <- bootS$quant[1, ,"x2"]
      ui2b.sim.e1[s, , c] <- bootS$quant[2, ,"x2"]
      cat("\n", "Bootstrapping took: ", my.time(proc.time() - sbt), "\n")
      boot.time.e1[s, c] <- (proc.time() - sbt)[3]
      cat("\n", "Mean boot for ce x1: ", mean(bootS$mean[ ,"x1"]), "\n")
      cat("\n", "Mean boot for se of ce x1: ", mean(bootS$se[ ,"x1"]), "\n")
    }, error = function(e) {cat("ERROR :", conditionMessage(e), "\n")}
    )
  }
}

#================================================
# Experiment 2: N = 1000, Boot = 10000 , Classes = 2 ----
#===============================================


## Globals

N <- 1000                     # Number of individuals 
J <- 3                        # Number of alternatives as Revelt & Train (2000)
times <- c(1, 5, 10, 20, 50)  # Choice situation per individual as Revelt & Train (2000)
S <- 300                      # Number of data bases
Q <- 1000                     # Number of Boot for Krisky Robb procedure.



## Generate True Random Coefficients: 
b1_e2 <- c(rep(-2,   N * 0.6),   rep(2,   N * 0.4))
b2_e2 <- c(rep(-0.5, N * 0.6),   rep(0.5, N * 0.4))


## Matrix for results b11 class 1 var 1, b12 class 1, var2
b11.sim.e2 <- matrix(NA, nrow = S, ncol = length(times))
b12.sim.e2 <- matrix(NA, nrow = S, ncol = length(times))
b21.sim.e2 <- matrix(NA, nrow = S, ncol = length(times))
b22.sim.e2 <- matrix(NA, nrow = S, ncol = length(times))
con2.sim.e2 <- matrix(NA, nrow = S, ncol = length(times))
share1.e2 <- matrix(NA, nrow = S, ncol = length(times))
share2.e2 <- matrix(NA, nrow = S, ncol = length(times))


bi1.sim.e2 <- array(NA, dim = c(S, N, ncol = length(times)))
bi2.sim.e2 <- array(NA, dim = c(S, N, ncol = length(times)))


si1.sim.e2 <- array(NA, dim = c(S, N, ncol = length(times)))
si2.sim.e2 <- array(NA, dim = c(S, N, ncol = length(times)))


bi1b.sim.e2 <- array(NA, dim = c(S, N, ncol = length(times)))
bi2b.sim.e2 <- array(NA, dim = c(S, N, ncol = length(times)))

si1b.sim.e2 <- li1b.sim.e2 <- ui1b.sim.e2  <- array(NA, dim = c(S, N, ncol = length(times)))
si2b.sim.e2 <- li2b.sim.e2 <- ui2b.sim.e2  <- array(NA, dim = c(S, N, ncol = length(times)))




mix.time.e2  <- matrix(NA, nrow = S, ncol = length(times))
boot.time.e2 <- matrix(NA, nrow = S, ncol = length(times))
mess.e2      <- matrix(NA, nrow = S, ncol = length(times))


## Monte Carlo Loop
for (c in 1:length(times)) {
  # Choice situation
  t <- times[c]
  
  # id for individuals
  id <- rep(1:N, each = J * t)
  
  for (s in 1:S) {
    cat("\n","==========================", "\n")
    cat("Time: ", t, ", and database: ", s, "\n")
    cat("===============================", "\n")
    
    # Generate variables that depend on T
    x1 <- rnorm(N * J * t)
    x2 <- as.numeric(runif(N * J * t) < 0.5)
    
    # The true data generating process
    U <-  b1_e2[id] * x1 + b2_e2[id] * x2 - log(-log(runif(N * J * t))) 
    
    ## Chosen alternative
    chosen <- rep(0, N * J * t)
    for (i in 1:(N * t)) {
      U_j <- U[((i - 1) * J + 1):(i * J)]
      U_max <- max(U_j)
      pos <- which(U_j == U_max)
      chosen[((i - 1) * J + 1):(i * J)][pos] <- TRUE
    }
    
    ## Data format
    data <- as.data.frame(cbind(id, chosen, x1, x2))
    data <- mlogit.data(data, choice = "chosen", shape = "long", 
                        alt.levels = c("1", "2", "3"), id.var = "id")
    # Estimate LC-MNL
    stm <- proc.time()
    tryCatch({
    if (t == 1) {
      lc <- gmnl(chosen ~ x1 + x2 | 0 | 0 | 0 | 1, 
                 data = data,
                 model = "lc",
                 print.init = TRUE,
                 Q = 2,  
                 seed = NULL,
                 iterlim = 1000)
    } else {
      lc <- gmnl(chosen ~ x1 + x2  | 0 | 0 | 0 | 1, 
                 data = data,
                 model = "lc",
                 print.init = TRUE,
                 Q = 2,  
                 seed = NULL,
                 iterlim = 1000,
                 panel = TRUE)
    }
    mix.time.e2[s, c] <- (proc.time() - stm)[3]
    cat("\n", "Estimation took: ", my.time(proc.time() - stm), "\n")
    
    cat("\n", "Coefficients are : ", "\n")
    print(coef(lc))
    summary(lc)
    mess.e2[s, c] <- lc$logLik$message
    
    cat("\n", "How is my LL doing?: ", mess.e2[s, c], "\n")
    b11.sim.e2[s, c] <- coef(lc)[1]
    b12.sim.e2[s, c] <- coef(lc)[2]
    b21.sim.e2[s, c] <- coef(lc)[3]
    b22.sim.e2[s, c] <- coef(lc)[4]
    con2.sim.e2[s, c] <- coef(lc)[5]
    
    ## Compute probability for each class
    share1.e2[s, c] <- exp(0) / (exp(0) + exp(con2.sim.e2[s, c]))
    share2.e2[s, c] <- exp(con2.sim.e2[s, c]) / (exp(0) + exp(con2.sim.e2[s, c]))
    
    
    ## Compute individual-estimates
    bq  <- lc$bi
    Qiq <- lc$Qir
    N   <- nrow(Qiq)
    K   <- ncol(bq)
    var_name <- colnames(bq)

    mean <- matrix(NA, N, K)
    se   <- matrix(NA, N, K)   
    colnames(mean) <- colnames(se) <- colnames(bq)
    for (j in 1:K) {
      b_temp <- gmnl:::repRows(bq[, j], N)
      mean[, j] <- rowSums(b_temp * Qiq)
      se[, j]   <- sqrt(rowSums((b_temp - mean[, j])^2 * Qiq))
    }
    
    
    bi1.sim.e2[s, , c] <- mean[, "x1"]
    bi2.sim.e2[s, , c] <- mean[, "x2"]
    si1.sim.e2[s, , c] <- se[, "x1"]
    si2.sim.e2[s, , c] <- se[, "x2"]
    
    # Bootstrapping
    sbt <- proc.time()

      if (t == 1) bootS <- boot.lc(lc, panel =  FALSE, S = Q, Q = 2) else bootS <- boot.lc(lc, panel =  TRUE, S = Q, Q = 2) 
      #bootS <- boot.lc(lc, panel =  TRUE, S = Q, Q = 2)
      bi1b.sim.e2[s, , c] <- bootS$mean[ ,"x1"]
      bi2b.sim.e2[s, , c] <- bootS$mean[ ,"x2"]
      si1b.sim.e2[s, , c] <- bootS$se[ ,"x1"]
      si2b.sim.e2[s, , c] <- bootS$se[ ,"x2"]
      li1b.sim.e2[s, , c] <- bootS$quant[1, ,"x1"]
      ui1b.sim.e2[s, , c] <- bootS$quant[2, ,"x1"]
      li2b.sim.e2[s, , c] <- bootS$quant[1, ,"x2"]
      ui2b.sim.e2[s, , c] <- bootS$quant[2, ,"x2"]
      cat("\n", "Bootstrapping took: ", my.time(proc.time() - sbt), "\n")
      boot.time.e2[s, c] <- (proc.time() - sbt)[3]
      cat("\n", "Mean boot for ce x1: ", mean(bootS$mean[ ,"x1"]), "\n")
      cat("\n", "Mean boot for se of ce x1: ", mean(bootS$se[ ,"x1"]), "\n")
    }, error = function(e) {cat("ERROR :", conditionMessage(e), "\n")}
    )
  }
}

save.image(file = "Data/LC_experimentAll.RData")

